/*******************************************************************************
*    Robot "Giskard" 1.0.                                                      *
*    Copyright (C) 2009 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Circuits by Marian Neubert marian@tesla-crew.de                           *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#ifndef NEWCOMM_H_
#define NEWCOMM_H_

#define boolean             unsigned short int
#define _POSIX_SOURCE       1
#define FALSE               0
#define TRUE                1
enum TYPE {INT, STRING, BOOL};

// Defaultwerte
#define DEFAULTCOMMFILE     "/var/run/rs232data"
#define DEFAULTANSWERFILE   "/var/run/rs232in"
#define DEFAULTPIDFILE      "/var/run/newcomm.pid"
#define DEFAULTLOGFILE      "/var/log/newcomm.log"
#define DEFAULTMODEMDEVICE  "/dev/ttyS0"
#define DEFAULTBAUDRATE     "9600"

// Programmdaten
#define MYPROTOCOLVERSION   2
#define MAXQUEUE            10 // Max 10 Befehl in der Queue
#define MAXDATALENGTH       100 // Max 100 Zeichen für die Daten
#define NAME                "newcomm"
#define VERSION             "1.0 RC1 Build 20090907"
#define TIMEOUT_SERIAL      3  // 3 Sekunden TimeOut
#define ACK_TIMEOUT         5  // 5 Sekunden TimeOut wenn ACK/NACK gewartet
#define MAX_RETRIES         3  // Das Paket wird maximal 3 mal geschickt im Falle vom NACK

// Loginformationen
#define LEVELERROR          0
#define LEVELINFO           1
#define LEVELDEBUG          2
char levelStr[3][255]       = {"ERROR", "INFO", "DEBUG"};

// Mögliche Antworten: ACK/NACK/PR_MISMATCH
unsigned char ACK           = 0x0F;
unsigned char NACK          = 0xF0;
unsigned char PR_MISMATCH   = 0xAA;

// Es wird auf TRUE gesetzt, wenn das Programm beendet werden muß (siehe quitFunc)
boolean STOP_PROGRAM        = FALSE;

// Struktur für den Datentausch µC <-> Laptop
struct dataFromRS232
{
  int version;                       // 1 Byte. Festkodiert MYPROTOCOLVERSION
  int deviceFrom;                    // 1 Byte
  int deviceTo;                      // 1 Byte
  int packetSize;                    // 1 Byte
  char data[MAXDATALENGTH];          // Befehl + Parameter, bzw. Antwort
  unsigned char parity;              // 1 Byte
};

// Struktur für eine Option vom Kommandozeile
struct option
{
  char *optNameLong;     // Name der Option
  char optNameShort;     // Name der Option (abgekürzt)
  enum TYPE type;        // Typ der Option (siehe enum TYPE)
  void *value;           // Gelesener Wert
  void *defValue;        // Defaultwert
};

// Struktur für alle Optionen vom Kommandozeile
struct options
{
  int nOptions;           // Anzahl der möglichen Optionen
  struct option* opt;     // Array mit Struktur der Optionen
  char *invalidOptionMsg; // Text für die ungültige angegebene Option
};

int hasToDebug, nodetach;
char pidfile[255];
char logfile[255];
char commfile[255];
char answerfile[255];
char modemDevice[255];
char baud[255];
speed_t baudRate;
int wait_ACK = FALSE;
int fd;
time_t lastStartTime, waitACKStartTime;
struct dataFromRS232 rs232Queue[MAXQUEUE];
int rs232QueueCount;
int rs232ErrorCount;
unsigned short int calculatedParity;

/**
 * Liest die Optionen vom Kommandozeile
 *
 * @param int            argc  Anzahl der Parameter im Kommandozeile
 * @param char*          argv  Die Parameter im Kommandozeile
 * @param struct options opts  Die Beschreibung der möglichen Parameter
 */
int getOptions(int argc, char* argv[], struct options opts);

/**
 * Startet das Programm als Daemon
 *
 * @param char* pidfile  Die Datei mit dem PID
 */
void daemonize(char *pidfile);

/**
 * Loggt was das Programm empfangen hat in die Datei logfile
 *
 * @param char* str     Die String zu loggen
 * @param int   level   Das Log-Level
 */
void logger(char *str, int level);

/**
 * Öffnet die serielle Schnittstelle, anhand der
 * angegebenen Parametern
 *
 * @return   0 wenn alles in Ordnung, -1 im Falle vom Probleme
 */
int openPort(void);

/**
 * Liest die Daten vom PIPE und serielle Schnittstelle.
 * Schickt die Daten dem µC und verwaltet die Antworten
 *
 * @param char* pidfile   Die Datei mit dem PID
 * @param int   nodetach  TRUE wenn das Programm in der Console laufen soll
 *                        FALSE wenn das Program als Daemon läuft
 */
void doWork(char *pidfile, int nodetach);

/**
 * Trapfunktion für CTRL+c.
 * Endet das Programm.
 *
 * @param int signum    Wird ignoriert, muß aber angegeben werden
 */
void quitFunc(int signum);

/**
 * Prüft, ob die Parität der empfangenen Daten stimmt
 *
 * @param struct dataFromRS232 d  Die empfangenen Daten
 * @return                        TRUE wenn stimmt, FALS ansonsten
 */
boolean checkParity(struct dataFromRS232 d);

/**
 * Rechnet die Parität des zu sendenden Pakets und fügt sie
 * in das Paket hinzu
 *
 * @param struct dataFromRS232 d  Die zu sendenden Daten
 */
void addParityToPacket(struct dataFromRS232 *d);

/**
 * Legt ein Datenpaket an
 *
 * @param int   fromDev           Absendergerät
 * @param int   toDev             Empfängergerät
 * @param char* d                 Die Daten
 * @return struct dataFromRS232   Das Datenpaket
 */
struct dataFromRS232 createData(int fromDev, int toDev, char *d);

/**
 * Fügt die Daten in die Queue, um sie zu senden
 *
 * @param struct dataFromRS232 d  Die zu sendenden Daten
 * @return boolean                TRUE wenn alles in Ordnung, FALSE wenn Problemen
 */
boolean addDataToQueue(struct dataFromRS232 d);

/**
 * Schickt die Daten, die in der Queue sind
 */
void flushQueue(void);

/**
 * Schickt ein ACK
 */
void sendSigACK(void);

/**
 * Schickt ein NACK
 */
void sendSigNACK(void);

/**
 * Schickt ein PR_MISMATCH
 */
void sendSigPRMismatch(void);

/**
 * Verwaltet die Daten vom PIPE an das µC
 *
 * @param char* buf   Die Daten, in dem Format:
 *                      Absender|Empfänger|Daten
 */
void managePIPE(char *buf);

/**
 * Holt n Bytes vom seriellen Schnittstelle
 *
 * @param int   fd    Der FileDescriptor der TTY
 * @param char* buf   Puffer, wo die Daten gespeichert werden
 * @param int   n     Anzahl der zu empfangenen Bytes
 * @return int        Anzahl der empfangenen Bytes
 */
int getNBytes(int fd, char *buf, int n);

/**
 * Verwaltet die Daten vom µC. Die Funktion wird aufgerufen, wenn ein Byte vom TTY ankommt
 *
 * @param char firstByte    Der erste Byte
 */
void manageRS232(char firstByte);

/**
 * Loggt die empfangenen Daten vom µC
 *
 * @param struct dataFromRS232 data  Die Daten
 */
void printData(struct dataFromRS232 data);

/**
 * Zeigt die Hilfe über das Programm
 */
void printHelp(void);

/**
 * Setzt die Geschwindigkeit der RS232
 *
 * @param char* baud    Die Geschwindigkeit als String.
 *                      Mögliche Werte: 9600, 19200, 38400, 57600, 115200
 * @return int          0 wenn alles in Ordnung, -1 falls Problemen
 */
int setBaudRate(char *baud);

#endif /* NEWCOMM_H_ */
